#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* These three Unix-specific header files are just for the call to
   "stat" in slurp_input that we use to determine how large a file
   is. */
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

/**** Part A */

/* TODO: declare a struct for binary search tree nodes */

/* TODO: make a function to create a binary search tree */

/* TODO: make a funciton to check whether any prefix of a string
   exists in a binary search tree */

/**** Part B */

/* Read all of the contents of the file named FNAME into a newly
   allocated character buffer, and put a pointer to the buffer at the
   location pointed to by CONTENTS_P. Returns the number of characters
   in the file. A newline is added to the end of the file if the file
   does not already end in a newline, and there is also a null
   character after the file contents. All the things that could go
   wrong related to I/O will cause the entire program to die, so other
   code doesn't need to worry about thandling them.
*/
long slurp_file(const char *fname, char **contents_p) {
    long res;
    struct stat stat_buf;
    long size;
    FILE *fh;
    char *buf;
    /* By determining the file size with stat(2), we don't need to use
       a dynamic data structure to store the contents, just a large
       flat array of the right size. */
    res = stat(fname, &stat_buf);
    if (res != 0) {
	fprintf(stderr, "Failed to access file %s: %s\n", fname,
		strerror(errno));
	exit(1);
    }
    size = stat_buf.st_size;
    buf = malloc(size + 2);
    if (!buf) {
	fprintf(stderr, "Failed to allocate memory for %s\n", fname);
	exit(1);
    }
    fh = fopen(fname, "r");
    if (!fh) {
	fprintf(stderr, "Failed to open %s for reading: %s\n", fname,
		strerror(errno));
	exit(1);
    }
    res = fread(buf, 1, size, fh);
    if (res != size) {
	fprintf(stderr, "Read only %ld bytes from %s\n", res, fname);
	exit(1);
    }
    res = fclose(fh);
    if (res) {
	fprintf(stderr, "Failure while closing %s: %s\n",
		fname, strerror(errno));
	exit(1);
    }
    if (buf[size - 1] != '\n') {
	/* Ensure the buffer is newline-terminated */
	buf[size++] = '\n';
    }
    /* Ensure the buffer is null-terminated */
    buf[size] = '\0';
    *contents_p = buf;
    return size;
}

/* TODO: make a qsort comparison function that compares the strings
   that two char *s point at. */

/* TODO: make a function that creates a blacklist binary search tree
   from a file. It should use slurp_file to read the contents, use
   qsort to sort the words, and call the creation function you
   implemented above. */

/**** Part C */

/* TODO: declare a struct for a node in a singly-linked list of strings */

/* TODO: implement code to print a user's From: line, but only if it
   hasn't been printed before, by using a linked list */

/* TODO: implement code to process messages, keeping track of
   responsible users and checking for forbidden words. */

int main(int argc, char **argv) {
    char *msg_buf;
    long msg_len;
    const char *blacklist_fname, *mbox_fname;
    
    /* Check that exactly two command-line arguments are supplied */
    if (argc != 3) {
	fprintf(stderr, "Usage: email-guardian <blacklist> <mailbox>\n");
	exit(1);
    }

    blacklist_fname = argv[1];
    mbox_fname = argv[2];
    
    /* TODO: read the blacklist from blacklist_fname */

    /* Read the email messages file */
    msg_len = slurp_file(mbox_fname, &msg_buf);

    /* TODO: scan the messages in msg_buf for blacklisted words, and
       print culprit users */

    /* If we got here, indicate successful completion: */
    return 0;
}
